package com.vp.plugin.sample.stereotypetags.actions;

import com.vp.plugin.*;
import com.vp.plugin.action.*;
import com.vp.plugin.model.*;
import com.vp.plugin.model.factory.*;

public class CreateStereotypeTagSampleActionController implements VPActionController {
	
	public void performAction(VPAction aAction) {
		
		IClass lClass1 = IModelElementFactory.instance().createClass();
		IClass lClass2 = IModelElementFactory.instance().createClass();
		
		// Find or create Stereotype
		IStereotype lStereotype = findOrCreateStereotype();
		
		if (lClass1.hasStereotype(lStereotype.getName())) {
			// this Use Case already have the stereotype
		}
		else {
			// Add stereotype. The Stereotype's TaggedValues will be auto assigned to this use case.
			lClass1.addStereotype(lStereotype);
			
			// set ClassTag's value as Actor
			ITaggedValueDefinition lClassTagDefinition = lStereotype.getTaggedValueDefinitions().getTaggedValueDefinitionByName("ClassTag");
			
			ITaggedValueContainer lTaggedValueContainer = lClass1.getTaggedValues();
			ITaggedValue[] lTaggedValues = lTaggedValueContainer.toTaggedValueArray();
			int lTaggedValueCount = lTaggedValues == null ? 0 : lTaggedValues.length;
			for (int lTaggedValueIndex = 0; lTaggedValueIndex < lTaggedValueCount; lTaggedValueIndex++) {
				ITaggedValue lTaggedValue = lTaggedValues[lTaggedValueIndex];
				if (
						lClassTagDefinition.equals(lTaggedValue.getTagDefinition()) && 
						"ClassTag".equals(lTaggedValue.getName())
				) {
					lTaggedValue.setValue(lClass2);
					break;
				}
			}
		}
		
	}
	
	public void update(VPAction aAction) {
	}
	
	private IStereotype findOrCreateStereotype() {

		IProject lProject = ApplicationManager.instance().getProjectManager().getProject();
		IModelElement[] lStereotypes = lProject.toAllLevelModelElementArray(IModelElementFactory.MODEL_TYPE_STEREOTYPE);
		int lStereotypeCount = lStereotypes == null ? 0 : lStereotypes.length;
		for (int lStereotypeIndex = 0; lStereotypeIndex < lStereotypeCount; lStereotypeIndex++) {
			IStereotype lStereotype = (IStereotype) lStereotypes[lStereotypeIndex];
			if (
					IModelElementFactory.MODEL_TYPE_CLASS.equals(lStereotype.getBaseType()) && 
					"OpenAPI_Stereotype".equals(lStereotype.getName())
			) {
				// Found (assume this stereotype is created by this plugin. So, it already contains the tags.) 
				return lStereotype;
			}
		}
		
		{
			// Create
			
			// create Stereotype
			IStereotype lStereotype = IModelElementFactory.instance().createStereotype();
			lStereotype.setName("OpenAPI_Stereotype");
			lStereotype.setBaseType(IModelElementFactory.MODEL_TYPE_CLASS);
			
			// create Stereotype.taggedValueDefinitions
			ITaggedValueDefinitionContainer lTaggedValueDefinitionContainer = lStereotype.getTaggedValueDefinitions();
			if (lTaggedValueDefinitionContainer == null) {
				lTaggedValueDefinitionContainer = IModelElementFactory.instance().createTaggedValueDefinitionContainer();
				lStereotype.setTaggedValueDefinitions(lTaggedValueDefinitionContainer);
			}
			{
				// - text (name=StringTag, default value="Hello")
				ITaggedValueDefinition lTaggedValueDefinition = lTaggedValueDefinitionContainer.createTaggedValueDefinition();
				lTaggedValueDefinition.setName("StringTag");
				lTaggedValueDefinition.setType(ITaggedValueDefinition.TYPE_STRING);
				lTaggedValueDefinition.setDefaultValue("Hello");
			}
			{
				// - enum (name=EnumTag, "value1", "value2" & "value3"), make value2 as default
				ITaggedValueDefinition lTaggedValueDefinition = lTaggedValueDefinitionContainer.createTaggedValueDefinition();
				lTaggedValueDefinition.setName("EnumTag");
				lTaggedValueDefinition.setType(ITaggedValueDefinition.TYPE_ENUMERATION);
				lTaggedValueDefinition.setEnumerationValues(new String[] {"value1", "value2", "value3"});
				lTaggedValueDefinition.setDefaultValue("value2");
			}
			{
				// - model element (name="ClassTag", value=the class created in step1)
				ITaggedValueDefinition lTaggedValueDefinition = lTaggedValueDefinitionContainer.createTaggedValueDefinition();
				lTaggedValueDefinition.setName("ClassTag");
				lTaggedValueDefinition.setType(ITaggedValueDefinition.TYPE_MODEL_ELEMENT);
//				lTaggedValueDefinition.setDefaultValue(...); // Default Value is not supported if type = Model Element
			}
			return lStereotype;
		}
		
	}

}
